#!/usr/bin/perl -w

=head1 NAME

B<POWER::lib> - Bundle of useful procedures.

=head1 SYNOPSIS

 use POWER::lib qw(time);

 $sec_with_microsec = time();

 $unsafe = "Programming Perl";
 $safe = EncUri($unsafe); 
 # $safe eq "Programming%20Perl"
 $safe = EncUri($unsafe, qr/[a-zA-Z]/);
 $decoded = UnEncUri($safe);

 $html_unsafe = "test < & >";
 $html = Enc($html_unsafe); 
 # $html eq "test &lt; &amp; &gt;"
 $text = UnEnc($html);

 $passwd = RandCh(8);

 $abs_url = abs_url("qwe.html", "http://host.com/dir1/a.html");
 # $abs_url eq "http://host.com/dir1/qwe.html"
 $abs_url = abs_url("/qwe.html", "http://host.com/dir1/a.html");
 # $abs_url eq "http://host.com/qwe.html"
 $abs_url = abs_url("http://host2.com/qwe.html",
    "http://host.com/dir1/a.html");
 # $abs_url eq "http://host2.com/qwe.html"

 print "$email looks good\n" if Check_Email($email);

 Email($template, \%fields);

=head1 DESCRIPTION

This a bundle of useful procedures. 
Procedures imported by default: EncUri(), UnEncUri(), Enc(), UnEnc(),
RandCh(), abs_url(), Check_Email(), Email().
Procedures imported by request: time().

=head1 FUNCTIONS

=over 4

=cut

package POWER::lib;
use MIME::Base64;
use strict;
use vars qw($VERSION);
$VERSION = "1.10" ;

sub import {
    no strict;
    *{"${\scalar caller}::$_"} = \&{$_} for qw(EncUri UnEncUri Enc UnEnc 
	RandCh abs_url Check_Email Email), @_;	
}

=item B<EncUri>( [$to_escape [, $regexp]] )

EncUri() replaces unsafe URI characters in $to_escape to his safe URI
representation in format %XX, where XX is character's code in hex.

$regexp can be used to define unsafe characters in form: qr/[ABC\d\s]/ .
By default unsafe characters are defined as qr/\W/ .

If $to_escape omitted $_ will be used instead.

Return escaped string.

=item B<UnEncUri>( [$to_unescape] )

UnEncUri() replaces all %XX assertions in $to_unescape to characters 
with hex code XX.

If $to_unescape omitted $_ will be used instead.

Return unescaped string.

=cut

my %EncUri = map { chr($_), sprintf("%%%02X",$_) } 0..255;
sub EncUri {
    my $to_escape =  ($#_ == -1) ? $_ : $_[0];
    my $regexp    =  $_[1] || qr/\W/; 
    $to_escape    =~ s/($regexp)/$EncUri{$1}/g;
    return $to_escape;
}
sub UnEncUri {
    my $to_unescape =  ($#_ == -1) ? $_ : $_[0];
    $to_unescape    =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
    return $to_unescape;
}

=item B<Enc>( [$html_unsafe] )

Enc() replaces unsafe HTML characters in $html_unsafe to HTML-entities.

If $html_unsafe omitted $_ will be used instead.

Return safe HTML string.

=item B<UnEnc>( [$html] )

UnEnc() decode all HTML entities.

If $html omitted $_ will be used instead.

Return decoded string.

=cut

my %Enc=(qw'& &amp; > &gt; < &lt; " &quot;',map{chr,"&#$_;"}(0..8,11..31,127,139,155));
sub Enc {
    my $html_unsafe =  ($#_ == -1) ? $_ : $_[0];
    $html_unsafe    =~ s/([\&\>\<\"\0-\10\13-\37\177])/$Enc{$1}/gs;
    return $html_unsafe;
}
my %UnEnc=reverse(qw'& &amp; > &gt; < &lt; " &quot;',map{chr,"&#$_;"}(0..255));
sub UnEnc {
    my $html =  ($#_ == -1) ? $_ : $_[0];
    $html    =~ s/(&(?:#\d+|\w+);)/$UnEnc{$1}/gs;
    return $html;
}

=item B<RandCh>( $length [, @chars] )

RandCh() return string filled by $length random characters.
The random characters are taken from ("A".."Z","a".."z",0..9,"_") 
or from @chars if specified.

=cut

# from "Perl Cooking Book"
my @RandCh = ( "A".."Z", "a".."z", 0..9, "_" );
sub RandCh {
    my $length = shift;
    my @chars  = @_ ? (@_) : (@RandCh);
    join("", @chars[ map { rand @chars } (1..$length) ]);
}

=item B<abs_url>( $relative_url, $base_url )

Construct and return new absolute url based on $relative_url and $base_url.

Was written as fast analog for 
URI->new_abs($relative_url, $base_url)->as_string() .

=cut

sub abs_url {
    my ($new, $base) = @_;
    #  ,    , ..  
    return $new if $new =~ /^\w+:/;
    #       
    $base =~ s!^([^?]+/).*!$1!;
    #          
    $base =~ s{([\w.]/).*$}{$1} if $new =~ m!^/!;
    $base .= $new;
    #     "" : "/./"
    $base =~ s!/\./!/!g;
    #   ""   : "/parent/../"
    1 while $base =~ s!(/(?:[\w-]+|\.[^.])+/)(?:[\w-]+|\.[^.])+/\.\./!$1!g;
    #      : "//"
    1 while $base =~ s!^([^?]+[^:])//+!$1/!g;
    return $base;
}

=item B<Check_Email>( $email )

Check syntax in $email. As minimum requred "user@host".

Return true if syntax looks ok.

=cut

sub Check_Email { no locale; $_[0]=~/^[\w\.-]+@[\w\.-]+$/ && not $_[0]=~/\.\./ }

=item B<Email>( $template, $data [, $attach] )

Send email.

If $template looks like a file name than email template will be read from it.
In other case scalar $template will be treated as template text.

$data must be a hash reference, and this hash will be used to substitute
fields in email template.
Fields format is: @~field_name~@ .

$attach must be a hash reference. Keys in this hash treated as file names.
If %$attach element has defined value this value used as file content, else
file content is read from disk.

After template is processed it is sent to mail program.
Mail program usually is "/usr/bin/sendmail -t".

Example:

 $template = <<EOT;
 From: Email robot <nobody>
 To: @~email~@
 Subject: @~subj~@

 Dear @~name~@.
 This message is a test.
 
 Sincerely, Robot.
 EOT
 $data = {
    email => 'user@remote.host.com',
    subj  => "Test",
    name  => "John Doe",
    };
 $attach = {
    "outbound/pic/1.jpg" => undef,
    "comment.txt" => "1.jpg is my photo.\n\nJonh.",
    };
 Email($template, $data, $attach);

=cut

my $Mail_Prog = "/usr/sbin/sendmail -t";# -oi -odq
sub Email {
    my ($file, $hash_ref, $attach) = @_;
    local (*MAIL, *TEXT);
    open(MAIL,"| $Mail_Prog") or die "Email: $!";
    local $_;
    if ($file =~ /\n/) {
	$_ = $file;
    } else {
	open(TEXT, $file) or die "open `$file': $!";
	local $/ = undef;
	$_ = <TEXT>;
    }
    s/@~(.*?)~@/$hash_ref->{$1}/gs;
    if (keys %$attach) {
	my ($head, $body) = split "\n\n",$_,2;
	my $bound = RandCh(35);
	$head .= "\nMIME-Version: 1.0\n".
	    "Content-Type: MULTIPART/MIXED; BOUNDARY=\"$bound\"\n\n";
	$body = "--$bound\n".
	    "Content-Type: TEXT/PLAIN; charset=US-ASCII\n\n".
	    $body."\n";
	for $file (keys %$attach) {
	    unless (defined $attach->{$file}) {
		local ($/, *F);
		open(F, $file) or die "open `$file': $!";
		$attach->{$file} = <F>;
	    }
	    (my $filename = $file) =~ s!.*/!!;
	    $body .= "--$bound\n".
		"Content-Type: APPLICATION/octet-stream; name=\"$filename\"\n".
		"Content-Transfer-Encoding: BASE64\n".
		"Content-Disposition: attachment; filename=\"$file\"\n\n".
		MIME::Base64::encode($attach->{$file});
	}
	$_ = $head . $body . "--$bound--\n";
    }
    print MAIL $_;
}

=item B<time>()

Return float time - seconds with microseconds. See `man gettimeofday`.
Work just like Time::HiRes, insignificantly slowly but don't require
compilation.

=cut

{ my $SYS_gettimeofday = sub {
	my @h = local @ARGV = grep {-r} map {"/usr/include/$_.h"}
		qw(syscall sys/syscall bits/syscall asm/unistd);
    /^\s*#define\s*(?:__NR|SYS)_gettimeofday\s*(\d+)/ and return $1 while <>;
    die "gettimeofday not found in: @h";
  }->();
sub time {
	syscall $SYS_gettimeofday, my $data=pack("ll"), 0;
	my ($sec, $msec) = unpack "ll", $data;
	return $sec+$msec/1000000;
}}

=back

=head1 HISTORY

 1.10       Mon Nov 12 07:22:56 EET 2001
       - added time()
 1.00		Tue Aug  7 19:40:32 EEST 2001
	   - first release
	
=head1 AUTHOR

 Alex Efros <powerman@consultant.com>
 Nikita Savin <nikita@only.com>

=cut

1;

